<?php
/**
 * This file is a part of EasyWatermark Wordpress plugin.
 * @see readme.txt
 */

/**
 * Main plugin class 
 */
class EasyWatermarkPlugin extends ewPluginCore
{
	/**
	 * @var string	plugin name
	 */
	protected static $pluginName = 'Easy Watermark';

	/**
	 * @var string	plugin slug used in setting names etc.
	 */
	protected static $pluginSlug = 'easy-watermark';

	/**
	 * @var string	plugin version
	 */
	protected static $version = '0.2.3';

	/**
	 * @var string	plugin class name
	 */
	protected static $className = 'EasyWatermarkPlugin';

	private $settings;

	private $messages = array();

	private $error = false;

	private $allowedMime = array('image/png', 'image/jpeg', 'image/gif');

	/**
 	 * Loads textdomain for translations,
 	 * adds wordpress actions and creates EasyWatermarkSettings object
	 *
	 * @return void
	 */
	public function __construct(){

		$this->add_action('init', 'plugin_init')
			->add_action('admin_menu', 'add_media_page')
			->add_filter('media_row_actions', 'add_media_row_action', 10, 3)
			->add_filter('attachment_fields_to_edit', 'add_attachment_field', 10, 2)
			->add_action('add_attachment', 'add_watermark_after_upload')
			->add_action('admin_notices')
			->add_action('admin_head-upload.php', 'add_bulk_action_script')
			->add_action('admin_action_ew_add_watermark', 'bulk_action_handler');
	}

	/**
 	 * Performs some actions that need to run before anything else
	 *
	 * @return void
	 */
	public function plugin_init(){
		new EasyWatermarkSettings($this);

		if(isset($_GET['page'])){
			if($_GET['page'] == 'easy-watermark-settings' && isset($_GET['tp']) && $_GET['tp'] == 1){
				$this->print_text_preview();
			}
			elseif($_GET['page'] == 'easy-watermark' && isset($_GET['_wpnonce']) && wp_verify_nonce($_GET['_wpnonce'])){
				$this->add_watermark();
			}
		}
	}

	/**
 	 * Returns generated jpeg image with text preview to the browser.
	 * Used for settings page
	 *
	 * @return void
	 */
	private function print_text_preview(){
		$ew = $this->getEasyWatermark();
		if($this->settings['general']['watermark_type'] == '1'){
			$settings = $this->settings['text'];

			$fontFile = dirname(__FILE__) . DIRECTORY_SEPARATOR . 'fonts' . DIRECTORY_SEPARATOR . $settings['font'];
			if(file_exists($fontFile))
				$settings['font'] = $fontFile;

			$ew->textSet($settings);
		}

		if(isset($_GET['text']))
			$ew->textSet('text', $_GET['text']);
		if(isset($_GET['size']))
			$ew->textSet('size', $_GET['size']);
		if(isset($_GET['angle']))
			$ew->textSet('angle', $_GET['angle']);
		if(isset($_GET['color']))
			$ew->textSet('color', $_GET['color']);
		if(isset($_GET['opacity']))
			$ew->textSet('opacity', $_GET['opacity']);

		if(isset($_GET['font'])){
			$fontFile = dirname(__FILE__) . DIRECTORY_SEPARATOR . 'fonts' . DIRECTORY_SEPARATOR . $_GET['font'];
			if(file_exists($fontFile))
				$ew->textSet('font', $fontFile);
		}

		$ew->printTextPreview();
		exit;
	}

	/**
 	 * Performs watermarking the single image
	 *
	 * @return void
	 */
	private function add_watermark(){
		if(isset($_GET['attachment_id'])){
			$url = false;
			if($this->watermark_single($_GET['attachment_id'])){
				switch($_GET['r']){
					case 'library':
						$url = admin_url('upload.php?watermarked=true');
						break;
					case 'post':
						$url = admin_url('post.php?post='.$_GET['attachment_id'].'&action=edit&watermarked=1');
						break;
				}
			}
			else {
				switch($_GET['r']){
					case 'library':
						$url = admin_url('upload.php?ew_error='.$this->error);
						break;
					case 'post':
						$url = admin_url('post.php?post='.$_GET['attachment_id'].'&action=edit&ew_error='.$this->error);
						break;
				}
			}
			if($url){
				wp_redirect($url);
				exit;
			}
		}
	}

	/**
 	 * Prints admin notices
	 *
	 * @return void
	 */
	public function admin_notices(){
		if(isset($_GET['watermarked']) && $_GET['watermarked'] == 'true'):
		?>
		<div class="updated">
			<p><?php _e('Watermark successfully added.', 'easy-watermark'); ?></p>
		</div>
		<?php
		elseif(isset($_GET['ew_error'])):
			echo '<div class="error"><p>';
			switch($_GET['ew_error']):
				case '1':
					_e('Invalid mime type.', 'easy-watermark');
					break;
				case '2':
					_e('No watermark image selected and no watermark text set.', 'easy-watermark');
					echo ' <a href="'.admin_url('options-general.php?page=easy-watermark-settings').'">';
					_e('Go to settings page', 'easy-watermark');
					echo '</a>';
					break;
				default:
					_e('An error has occurred.', 'easy-watermark');
			endswitch;
			echo '</p></div>';
		endif;
	}

	/**
 	 * Adds wordpress media page
	 *
	 * @return void
	 */
	public function add_media_page(){
 		add_media_page( 'Easy Watermark', 'Easy Watermark', 'upload_files', 'easy-watermark', array($this, 'easy_watermark'));
	}

	function add_bulk_action_script() {
		?>
		<script type="text/javascript">
			jQuery(document).ready(function() {
				jQuery('<option>').val('ew_add_watermark').text('<?php _e('Add Watermark', 'easy-watermark')?>').appendTo("select[name='action']");
				jQuery('<option>').val('ew_add_watermark').text('<?php _e('Add Watermark', 'easy-watermark')?>').appendTo("select[name='action2']");
			});
		</script>
		<?php
	}

	/**
 	 * Creates 'Add watermark' link for each row in media library
	 *
	 * @return	array
	 */
	public function add_media_row_action($actions, $post, $detached){
		if($post->post_mime_type == 'image/jpeg' || $post->post_mime_type == 'image/png' || $post->post_mime_type == 'image/gif'){
			// Add link if it's supported image type
			$actions['add_watermark'] = '<a href="' . wp_nonce_url(admin_url('upload.php?page=easy-watermark&attachment_id='.$post->ID.'&r=library')) . '">'.__('Add watermark', 'easy-watermark').'</a>';
		}

		return $actions;
	}

	/**
 	 * Creates 'Add watermark' button in media edit view
	 *
	 * @return	array
	 */
	public function add_attachment_field($form_fields, $post){
		if($post->post_mime_type == 'image/jpeg' || $post->post_mime_type == 'image/png' || $post->post_mime_type == 'image/gif'){
			// Add link if it's supported image type
			$form_fields = array_reverse($form_fields);
			$form_fields['easy-watermark'] = array(
				'label' => '<a href="'.wp_nonce_url(admin_url('upload.php?page=easy-watermark&attachment_id='.$post->ID.'&r=post')).'" class="button-secondary">'.__('Add watermark', 'easy-watermark').'</a>',
				'input' => 'html',
				'html' => ' '
			);
		}

		return array_reverse($form_fields);
	}

	/**
 	 * Handles the ew_add_watermark bulk action
	 * Performs watermarking selected images
	 *
	 * @uses   self::watermark_single
	 * @return  void
	 */
	public function bulk_action_handler(){

		if(empty($_REQUEST['action']) || ($_REQUEST['action'] != 'ew_add_watermark' && $_REQUEST['action2'] != 'ew_add_watermark')){
			return;
		}

		if(empty($_REQUEST['media']) || !is_array($_REQUEST['media'])){
			return;
		}
		
		check_admin_referer('bulk-media');
		$result = true;
		foreach($_REQUEST['media'] as $entry){
			if(!$this->watermark_single((int) $entry))
				$result = false;
		}

		if(isset($_GET['_wp_http_referer'])){
			$referer = $_GET['_wp_http_referer'];
			if(strpos($referer, '?'))
				$referer .= '&';
			else
				$referer .= '?';

			$referer .= $result ? 'watermarked=true' : 'ew_error=2';

			wp_redirect($referer);
			exit;
		}
	}

	/**
 	 * Watermark image after it was uploaded.
	 * In fact this method only mark that there is an image to watermark,
	 * it's realy watermarked in wp_generate_attachment_metadata filter.
	 * See wp_generate_attachment_metadata() method below.
	 *
	 * @return	array
	 */
	public function add_watermark_after_upload($id){
		if($this->settings['general']['auto_add']){
			$this->watermark_uploaded = true;
			$this->uploaded_id = $id;
			$this->add_filter('wp_generate_attachment_metadata', null, 10, 2);
		}
	}

	/**
 	 * Filter used to add watermark
	 * add_attachment action is called before image resizing,
	 * so image was watermarked first, then resized.
	 * To avoid that, in add_attachment action we only mark,
	 * that image was uploaded and then it's watermarked in this filter
	 * which is called after resizing.
	 *
	 * @param	 array
	 * @param	 integer
	 * @return	array
	 */
	public function wp_generate_attachment_metadata($metadata, $id){
		if($this->watermark_uploaded && $this->uploaded_id == $id){
			$this->watermark_single($id, true);

			$this->watermark_uploaded = false;
			$this->uploaded_id = null;
		}

		return $metadata;
	}

	/**
 	 * Performs the action of a single image watermarking
	 *
	 * @uses   self::create_watermark
	 * @return boolean
	 */
	private function watermark_single($id, $checkMime = false){
		if($this->settings['image']['watermark_path'] != '' || $this->settings['text']['text'] != null){
			$post = get_post($id);
			$filepath = get_attached_file($id);
			if(!in_array($post->post_mime_type, $this->allowedMime)){
				$this->error = 1;
				return false;
			}
			if($checkMime && !in_array($post->post_mime_type, $this->settings['general']['image_types'])){
				return false;
			}

			return $this->create_watermark($filepath, $post->post_mime_type);
		}
		else {
			$this->error = 2;
			return false;
		}
	}

	/**
 	 * Performs the action of watermarking all images from library
	 *
	 * @uses   self::create_watermark
	 * @return boolean
	 */
	private function watermark_all(){
		global $wpdb;

		$images = $wpdb->get_results("
			SELECT ID, post_mime_type
			FROM $wpdb->posts
			WHERE post_type = 'attachment'
			AND post_mime_type LIKE 'image/%'
		");

		$output = '';
		if($images && ($this->settings['image']['watermark_path'] != '' || $this->settings['text']['text'])){
			foreach($images as $img){
				$filepath = get_attached_file($img->ID);
				if(!in_array($img->post_mime_type, $this->allowedMime)){
					$output .= sprintf(__('Not supported mime type of %s. Skipping...', 'easy-watermark'), '<strong>'.$filepath.'</strong>').'<br/>';
					continue;
				}

				if($this->create_watermark($filepath, $img->post_mime_type)){
					$output .= sprintf(__('Watermark successfully added to %s', 'easy-watermark'), '<strong>'.$filepath.'</strong>') . '<br/>';
				}
			}
		}
		else return false;

		$output = '<p>'.$output.'</p>';

		return $output;
	}

	/**
 	 * Applies the watermark to the given image
	 *
	 * @param  string path to image file
	 * @param  string image mime type
	 * @return boolean
	 */
	private function create_watermark($imageFile, $imageType){
		if(empty($this->settings['image']['watermark_path']) || $this->settings['image']['watermark_path'] != $imageFile){
			$ew = $this->getEasyWatermark();
			$ew->setImagePath($imageFile)
				->setImageMime($imageType)
				->setOutputFile($imageFile)
				->setOutputMime($imageType);

			if(!$ew->create() || !$ew->saveOutput()){
				$this->add_error($ew->getError());
				return false;
			}

			return true;
		}
		else {
			return false;
		}
	}

	/**
 	 * @var  object  stores EasyWatermark object
	 */
	private $ew;

	/**
 	 * Returns configured EasyWatermark object 
	 *
	 * @return object
	 */
	public function getEasyWatermark(){
		if(!($ew instanceof EasyWatermark)){
			$imageSettings = $this->settings['image'];
			$textSettings = $this->settings['text'];
			$ew = new EasyWatermark();

			$fontFile = dirname(__FILE__) . DIRECTORY_SEPARATOR . 'fonts' . DIRECTORY_SEPARATOR . $textSettings['font'];
			if(file_exists($fontFile))
				$textSettings['font'] = $fontFile;

			$wType = $this->settings['general']['watermark_type'];
			if($wType == 1 || $wType == 3)
				$ew->imageSet($imageSettings);
			if($wType == 2 || $wType == 3)
				$ew->textSet($textSettings);
		}

		return $ew;
	}

	/**
 	 * Prints Easy Watermark page
	 *
	 * @return void
	 */
	public function easy_watermark(){
		?>
		<div class="wrap easy-watermark">
			<h2><?php _e('Easy Watermark', 'easy-watermark'); ?></h2>
		<?php
			if(isset($_GET['_wpnonce']) && wp_verify_nonce($_GET['_wpnonce'])){
				if(isset($_GET['watermark_all'])){
					if($output = $this->watermark_all()) :
					?>
			<div id="message" class="updated below-h2">
				<p><?php _e('Watermark successfully added.', 'easy-watermark'); ?> <a href="<?php echo admin_url('upload.php') ?>"><?php _e('Go to Media Library', 'easy-watermark'); ?></a></p>
			</div>
					<?
						echo $output;
					endif;
				}
			}
			else {
			?>
			<br/>
			<?php require_once dirname(__FILE__) . '/donation.php';
			ewDonation::showButton(); ?>
			<a class="button-primary" href="<?php echo wp_nonce_url(admin_url('/upload.php?page=easy-watermark&watermark_all=1')); ?>"><?php _e('Add watermark to all images', 'easy-watermark'); ?></a><p class="description"><?php _e('Be carefull with that option. If some images alredy has watermark, it will be added though.', 'easy-watermark'); ?></p>
			<?php
			}
		?>
		</div>
		<?php
	}

	/**
 	 * Method run when activating plugin
	 *
	 * @return void
	 */
	public static function install($settings = null){
		if(empty($settings)) $settings = EasyWatermarkSettings::getDefaults();

		foreach($settings as $sectionName => $section){
			update_option(static::$pluginSlug.'-settings-'.$sectionName, $section);
		}
		update_option(static::$pluginSlug.'-version', static::$version);
	}

	/**
 	 * Method run when removing plugin
	 *
	 * @return void
	 */
	public static function uninstall(){
		$settings = EasyWatermarkSettings::getDefaults();

		foreach($settings as $sectionName => $section){
			delete_option(static::$pluginSlug.'-settings-'.$sectionName);
		}
		delete_option(static::$pluginSlug.'-version');
	}

	/**
 	 * Method run when plugin version stored in WP options
	 * is lower than current version.
	 *
	 * @param  string  previously installed version
	 * @return void
	 */
	protected static function upgrade($version){
		if($version == '0.2.2'){
			return;
		}
		elseif($version == '0.2' || $version == '0.2.1'){
			$settings['image'] = get_option(static::$pluginSlug.'-settings-image');
			$settings['text'] = get_option(static::$pluginSlug.'-settings-text');
		}
		else {
			$oldSettings = get_option(static::$pluginSlug.'-settings');
			$defaults = EasyWatermarkSettings::getDefaults();

			$imgTypes = array();
			foreach($oldSettings['image_types'] as $type){
				$imgTypes[] = $type;
			}

			$general = array(
				'auto_add' => $oldSettings['auto_add'],
				'image_types' => $imgTypes
			);

			switch($version){
				case '0.1.1':
					$image = array(
						'watermark_url' => $oldSettings['image']['url'],
						'watermark_id' => $oldSettings['image']['id'],
						'watermark_path' => $oldSettings['image']['path'],
						'watermark_mime' => $oldSettings['image']['mime'],
						'position_x' => $oldSettings['image']['position_x'],
						'position_y' => $oldSettings['image']['position_y'],
						'offset_x' => $oldSettings['image']['offset_x'],
						'offset_y' => $oldSettings['image']['offset_y'],
						'opacity' => $oldSettings['image']['opacity']
					);
					break;
				default:
					$image = array(
						'watermark_url' => $oldSettings['image']['url'],
						'watermark_id' => $oldSettings['image']['id'],
						'watermark_path' => $oldSettings['image']['path'],
						'watermark_mime' => $oldSettings['image']['mime'],
						'position_x' => $oldSettings['image']['position-horizontal'],
						'position_y' => $oldSettings['image']['position-vert'],
						'offset_x' => $oldSettings['image']['offset-horizontal'],
						'offset_y' => $oldSettings['image']['offset-vert'],
						'opacity' => $oldSettings['image']['alpha']
					);
					break;
			}

			$settings = array(
				'general' => array_merge($defaults['general'], $general),
				'image' => array_merge($defaults['image'], $image),
				'text' => $defaults['text']
			);
			delete_option(static::$pluginSlug.'-settings');
		}

		$settings['image']['alignment'] = static::getAlignment($settings['image']['position_x'], $settings['image']['position_y']);
		$settings['text']['alignment'] = static::getAlignment($settings['text']['position_x'], $settings['text']['position_y']);

		static::install($settings);
	}

	/**
 	 * Computes alignment number based on position_x and position_y
	 *
	 * @param  int
	 * @param  int
	 * @return int
	 */
	private static function getAlignment($x, $y){
		$a = false;
		switch($y){
			case 'top':
				switch($x){
					case 'lft':
					case 'left':
						$a = 1;
						break;
					case 'ctr':
					case 'center':
						$a = 2;
						break;
					case 'rgt':
					case 'right':
						$a = 3;
						break;
				}
				break;
			case 'mdl':
			case 'middle':
				switch($x){
					case 'lft':
					case 'left':
						$a = 4;
						break;
					case 'ctr':
					case 'center':
						$a = 5;
						break;
					case 'rgt':
					case 'right':
						$a = 6;
						break;
				}
				break;
			case 'btm':
			case 'bottom':
				switch($x){
					case 'lft':
					case 'left':
						$a = 7;
						break;
					case 'ctr':
					case 'center':
						$a = 8;
						break;
					case 'rgt':
					case 'right':
						$a = 9;
						break;
				}
				break;
		}
		
		return $a;
	}

	private function add_error($msg){
		$this->messages[] = array('error', $msg);
	}

	private function add_info($msg){
		$this->messages[] = array('update', $msg);
	}

	/**
 	 * Sets settings array.
	 *
	 * @return void
	 */
	public function setSettings($settings){
		$this->settings = $settings;
	}
}
