<?php

class EasyWatermarkSettings
{
	private $settings = array();

	private $plugin;

	private static $defaults = array(
		'general' => array(
			'auto_add' => '1',
			'image_types' => array('image/jpeg', 'image/png', 'image/gif'),
			'watermark_type' => 3
		),
		'image' => array(
			'watermark_url' => null,
			'watermark_id' => null,
			'watermark_path' => null,
			'watermark_mime' => null,
			'position_x' => 'center',
			'position_y' => 'middle',
			'alignment' => 5,
			'offset_x' => 100,
			'offset_y' => 100,
			'opacity' => 100
		),
		'text' => array(
			'position_x' => 'center',
			'position_y' => 'middle',
			'alignment' => 5,
			'offset_x' => 0,
			'offset_y' => 0,
			'opacity' => 60,
			'color' => '000000',
			'font' => 'Arial.ttf',
			'size' => 24,
			'angle' => 0,
			'text' => ''
		)
	);

	private $tabs;

	private $fonts = array(
			'Arial.ttf' => 'Arial',
			'Arial_Black.ttf' => 'Arial Black',
			'Comic_Sans_MS.ttf' => 'Comic Sans MS',
			'Courier_New.ttf' => 'Courier New',
			'Georgia.ttf' => 'Georgia',
			'Impact.ttf' => 'Impact',
			'Tahoma.ttf' => 'Tahoma',
			'Times_New_Roman.ttf' => 'Times New Roman',
			'Trebuchet_MS.ttf' => 'Trebuchet MS',
			'Verdana.ttf' => 'Verdana',
	);

	public static function getDefaults(){
		return self::$defaults;
	}

	public function __construct($plugin){

		add_action('admin_menu', array($this, 'add_options_page'));
		add_action('admin_init', array($this, 'register_settings'));
		add_filter('plugin_action_links', array($this, 'settings_link'), 10, 2);
		add_filter('plugin_row_meta', array($this, 'plugin_row_meta'), 10, 2);

		$this->tabs = array(
			'general' => __('General', 'easy-watermark'),
			'image' => __('Image', 'easy-watermark'),
			'text' => __('Text', 'easy-watermark'),
		);

		foreach($this->tabs as $name => $caption){
			$section = get_option($plugin->getSlug() . '-settings-' . $name);
			$this->settings[$name] = array_merge(self::$defaults[$name], $section);
		}

		$this->plugin = $plugin;

		$plugin->setSettings($this->settings);
	}

	public function get($key){
		return $this->options[$key];
	}

	public function add_options_page(){
 		add_options_page(__('Easy Watermark', 'easy-watermark'), __('Easy Watermark', 'easy-watermark'), 'manage_options', 'easy-watermark-settings', array($this, 'settings_page'));
	}

	public function register_settings(){
		foreach($this->tabs as $name => $caption){
			register_setting(
				$this->plugin->getSlug() . '-settings-' . $name,
				$this->plugin->getSlug() . '-settings-' . $name,
				array($this, 'sanitize_settings_' . $name)
			);
		}
	}

	public function sanitize_settings_general($input){
		if(!isset($input['auto_add']) || $input['auto_add'] !== '1'){
			$input['auto_add'] = false;
		}

		if(!isset($input['image_types'])){
			$input['image_types'] = array();
		}

		$input = wp_parse_args($input, $this->options);

		return $input;
	}

	public function sanitize_settings_image($input){

		if(!empty($input['watermark_url'])){
			if(isset($input['old-manager'])){
				// old wordpress media library, we have only image url
				global $wpdb;

				$row = $wpdb->get_row("
					SELECT ID, post_mime_type
					FROM $wpdb->posts
					WHERE $wpdb->posts.post_type = 'attachment'
					AND $wpdb->posts.guid = '{$input['image']['url']}'
				");

				$input['watermark_id'] = $row->ID;
				$input['watermark_mime'] = $row->post_mime_type;
				unset($input['old-manager']);
			}

			$input['watermark_path'] = get_attached_file($input['watermark_id']);
		}

		switch($input['alignment']){
			case '1':
				$input['position_x'] = 'left';
				$input['position_y'] = 'top';
				break;
			case '2':
				$input['position_x'] = 'center';
				$input['position_y'] = 'top';
				break;
			case '3':
				$input['position_x'] = 'right';
				$input['position_y'] = 'top';
				break;
			case '4':
				$input['position_x'] = 'left';
				$input['position_y'] = 'middle';
				break;
			case '5':
				$input['position_x'] = 'center';
				$input['position_y'] = 'middle';
				break;
			case '6':
				$input['position_x'] = 'right';
				$input['position_y'] = 'middle';
				break;
			case '7':
				$input['position_x'] = 'left';
				$input['position_y'] = 'bottom';
				break;
			case '8':
				$input['position_x'] = 'center';
				$input['position_y'] = 'bottom';
				break;
			case '9':
				$input['position_x'] = 'right';
				$input['position_y'] = 'bottom';
				break;
		}

		return $input;
	}

	public function sanitize_settings_text($input){

		switch($input['alignment']){
			case '1':
				$input['position_x'] = 'left';
				$input['position_y'] = 'top';
				break;
			case '2':
				$input['position_x'] = 'center';
				$input['position_y'] = 'top';
				break;
			case '3':
				$input['position_x'] = 'right';
				$input['position_y'] = 'top';
				break;
			case '4':
				$input['position_x'] = 'left';
				$input['position_y'] = 'middle';
				break;
			case '5':
				$input['position_x'] = 'center';
				$input['position_y'] = 'middle';
				break;
			case '6':
				$input['position_x'] = 'right';
				$input['position_y'] = 'middle';
				break;
			case '7':
				$input['position_x'] = 'left';
				$input['position_y'] = 'bottom';
				break;
			case '8':
				$input['position_x'] = 'center';
				$input['position_y'] = 'bottom';
				break;
			case '9':
				$input['position_x'] = 'right';
				$input['position_y'] = 'bottom';
				break;
		}

		return $input;
	}

	public function settings_page(){
		wp_enqueue_script('ew-colorpicker', plugin_dir_url(dirname(__File__) . '/index.php') . 'js/colorpicker.js');
		wp_enqueue_script('ew-interface', plugin_dir_url(dirname(__File__) . '/index.php') . 'js/interface.js');
		wp_enqueue_style('ew-style', plugin_dir_url(dirname(__File__) . '/index.php') . 'css/style.css');
		wp_enqueue_style('ew-cp-style', plugin_dir_url(dirname(__File__) . '/index.php') . 'css/colorpicker.css');
		if(function_exists('wp_enqueue_media')){
			// load new media manager (since wp 3.5)
			wp_enqueue_media();
			wp_enqueue_script('ew-media-libraby', plugin_dir_url(dirname(__File__) . '/index.php') . 'js/media-library.js');
		}
		else {
			// load old-style thiskbox
			wp_enqueue_script('thickbox');
			wp_enqueue_style('thickbox');
			wp_enqueue_script('ew-media-libraby', plugin_dir_url(dirname(__File__) . '/index.php') . 'js/old-media-library.js');
		}

		$fonts = $this->fonts;

		$current_tab = (isset($_GET['tab']) && array_key_exists($_GET['tab'], $this->tabs)) ? $_GET['tab'] : 'general';
		extract($this->settings[$current_tab]);
		include dirname(__File__) . '/settings-form.php';
	}

	function settings_link($links, $file){
		static $this_plugin;
 
		if (!$this_plugin) {
			$this_plugin = plugin_basename(dirname(__FILE__) . '/index.php');
		}
		if ($file == $this_plugin) {
			$settings_link = '<a href="options-general.php?page=easy-watermark-settings">'.__('Settings').'</a>';
			array_unshift($links, $settings_link); 
		}
		return $links;
	}

	function plugin_row_meta($links, $file){
		static $this_plugin;
 
		if (!$this_plugin) {
			$this_plugin = plugin_basename(dirname(__FILE__) . '/index.php');
		}
		if ($file == $this_plugin) {
			require_once dirname(__FILE__) . '/donation.php';
			$donate_link = '<a href="'.ewDonation::getUrl().'">'.__('Donate', 'easy-watermark').'</a>';
			array_push($links, $donate_link); 
		}
		return $links;
	}
}
